/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2015 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************
* File Name    : r_IICA0_LIB.c
* Device(s)    : R5F10267
* Tool-Chain   : CC-RL
* Description  : This file implements device driver for IICA0 slave mode.
* Creation Date: 2015/11/09
************************************************************************/

/***********************************************************************
Includes
************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"

/***********************************************************************
Pragma directive
************************************************************************/
#pragma interrupt INTIICA0 r_iica0_interrupt

/***********************************************************************
Global variables , constant and functions
************************************************************************/
#define     DATA_MAX        ( 2 )           /* data number to transfer      */

#define     TX_LIMIT        ( DATA_MAX -1 ) /* transmit buffer mask data    */
#define     RX_LIMIT        ( DATA_MAX -1 ) /* receive buffer mask data     */
#define     RAM_BASE        ( 0x80 )        /* RAM is located 0x80 -0xFF    */
#define     RAM_MASK        ( RAM_BASE - 1 )/* RAM address mask data        */

#define     COM_FLAG_MASK   ( 0b00001111 )  /* communication flag clear data*/
#define     SLAVE_MODE      ( 0 )           /* start as slave mode          */
#define     DETECT          ( 1 )           /* detect flag                  */
#define     SELECTED        ( 1 )           /* selected by master           */
#define     NOT_SELECTED    ( 0 )           /* not selected by master       */
#define     RECEIVE_MODE    ( 0 )           /* IIC bus receive operation    */
#define     TRANSMIT_MODE   ( 1 )           /* IIC bus transmit operation   */
#define     NACK            ( 0 )           /* not ACK from master          */

uint8_t     static g_ram_area[128];         /* 128byte RAM area for master  */
uint8_t     static g_rx_data[DATA_MAX];     /* received data buffe          */
uint16_t    static g_tx_data[DATA_MAX];     /* transmit data buffer         */
uint8_t     static g_low_data_temp;         /* low byte data for transmit   */
uint8_t     static g_low_data_index;        /* show next transmit is low    */

uint8_t     static sreg gp_trx_data;        /* transfer data store pointer  */

uint8_t     static sreg g_status;           /* IICA0 status flag            */
#define     F_MODE          (g_status.7)    /* mode flag
                                            0:not communicate
                                            1:in communication              */
#define     F_DIR           (g_status.6)    /* transmit flag
                                            0:receive
                                            1:transmit                      */
#define     F_READY         (g_status.5)    /* communication ready flag
                                            0:not ready
                                            1:enable communication          */
#define     F_REGADR        (g_status.4)    /* register address timing
                                            0:not register address set
                                            1:register address timing       */
#define     F_SPD           (g_status.3)    /* detect STOP condition
                                            0:not detect
                                            1:detect STOP condition         */


uint8_t     static sreg g_IICS_f;           /* IICS0 data store buffer      */
#define     SPD_f           (g_IICS_f.0)    /* Stop condition detect flag   */
#define     STD_f           (g_IICS_f.1)    /* Start condition detect flag  */
#define     ACKD_f          (g_IICS_f.2)    /* ACK detect flag              */
#define     TRC_f           (g_IICS_f.3)    /* transmit flag                */
#define     COI_f           (g_IICS_f.4)    /* selected by address flag     */
#define     EXC_f           (g_IICS_f.5)    /* extension code reception flag*/
#define     ALD_f           (g_IICS_f.6)    /* arbitration loss flag        */
#define     MSTS_f          (g_IICS_f.7)    /* Master status flag           */


void R_IICA0_Init(void);                    /* IICA0 buffer initialize      */
uint8_t R_IICA0_Get(uint8_t ptr);           /* get IICA0 received data      */
void R_IICA0_Put(uint8_t ptr,uint16_t data); /* set data to TX buffer       */
uint8_t R_IICA0_Status(void);               /* check IICA0 status           */

/***********************************************************************
* Function Name: R_IICA0_Init
* Description  : This function is IICA0 data initialize routine.
* Arguments    : None
* Return Value : None
************************************************************************/
void R_IICA0_Init(void)
{
    uint8_t i;

    for( i = 0 ; i < DATA_MAX ; i++ )       /* clear data buffer            */
    {
        g_rx_data[i] = 0x00;
        g_tx_data[i] = 0x00;
    }

    for( i = 0 ; i < RAM_BASE ; i++ )
    {
        g_ram_area[i] = i;                  /* initialize RAM data          */
    }

    SPIE0 = TRUTH;                          /* enable STOP detect interrupt */
    gp_trx_data = 0x00;                     /* initialize buffer pointer    */

    g_low_data_index = 0x00;                /* next data is high byte data  */

}/* End of function R_IICA0_Init() */

/***********************************************************************
* Function Name: R_IICA0_Get
* Description  : This function get RX buffer data routine.
* Arguments    : pointer
* Return Value : data
************************************************************************/
uint8_t R_IICA0_Get(uint8_t ptr)
{

    uint8_t rx_data;

    rx_data = g_rx_data[(ptr & RX_LIMIT)];  /* read data from buffer      */
    return( rx_data );

}/* End of function R_IICA0_Get() */

/***********************************************************************
* Function Name: R_IICA0_Status
* Description  : This function get IICA0 status and clear STOP cond flag
* Arguments    : None
* Return Value : status
************************************************************************/
uint8_t R_IICA0_Status(void)
{

    uint8_t status;

    IICAMK0 = INT_MASK;                     /* for exclusive control        */
    status = g_status;                      /* read IICA0  status           */
    g_status &= ~DETECT_STOP;               /* clear STOP condition flag    */
    IICAMK0 = INT_ENABLE;                   /* enable INTIICA0              */

    return( status );

}/* End of function R_IICA0_Status() */


/***********************************************************************
* Function Name: R_IICA0_Put
* Description  : This function set data to TX buffer.
* Arguments    : pointer,data
* Return Value : None
************************************************************************/
void R_IICA0_Put(uint8_t ptr,uint16_t data)
{

    uint8_t wpt;

    wpt = ptr & TX_LIMIT;                   /* mask non used bit            */

    IICAMK0 = INT_MASK;                     /* for exclusive control        */
    g_tx_data[wpt] = data;                  /* store data in Tx buffer      */
    IICAMK0 = INT_ENABLE;                   /* enable INTIICA0              */

}/* End of function R_IICA0_Put() */


/***********************************************************************
* Function Name: r_iica0_interrupt
* Description  : This function is INTIICA0 interrupt service routine.
* Arguments    : None
* Return Value : None
************************************************************************/
__interrupt static void r_iica0_interrupt(void)
{

    uint16_t datawork;                      /* for transmit data (temp)     */

    g_IICS_f = IICS0;                       /* read IICA0 status            */

    if( SLAVE_MODE == MSTS_f)
    {
        if( DETECT == SPD_f )               /* check STOP condition         */
        {                                   /* STOP condition is detected   */

/*--------------------------------------
    STOP condition detect interrupt
--------------------------------------*/

            g_status &= COM_FLAG_MASK;      /* clear communication flag     */
            g_status |= DETECT_STOP;        /* set STOP condition flag      */

        }                                   /* and exit from communication  */
        else                                /* not detect STOP condition    */
        {

/*--------------------------------------
    data transfer end interrupt timing
--------------------------------------*/

            if( DETECT == STD_f )           /* check START condition        */
            {                               /* detect START condition       */
                if( NOT_SELECTED == COI_f ) /* check selected or not        */
                {                           /* not selected and exit        */
                    g_status &= COM_FLAG_MASK; /* clear communication flag  */
                    LREL0 = 1;              /* exit from communication      */
                }
                else
                {

/*--------------------------------------
    selected by slave address
--------------------------------------*/

                    F_DIR = TRC_f;          /* set transfer mode flag       */
                    F_READY = FALSE;        /* clear ready flag             */
                    F_MODE = TRUTH;         /* set on communication flag    */

                    if( RECEIVE_MODE == TRC_f ) /* check transmit or receive */
                    {

/*--------------------------------------
    receive mode start
    (start register address)
--------------------------------------*/
                        F_REGADR = TRUTH;   /* wait for register address  */
                        WREL0 = TRUTH;      /* Cancel wait & start receive  */

                    }
                    else
                    {

                        if ( RAM_BASE > gp_trx_data )
                        {
/*--------------------------------------
    data transmit mode start
    (transmit from high byte)
--------------------------------------*/

                            g_low_data_index = TRUTH; /* set low byet ready     */
                            datawork = g_tx_data[gp_trx_data];
                            IICA0 = (datawork >> 8);  /* transmit high byte     */
                            g_low_data_temp = datawork & 0xFF;
                            gp_trx_data++;
                            gp_trx_data &= TX_LIMIT;
                        }
                        else
                        {
/*--------------------------------------
    RAM data transmit mode start
--------------------------------------*/
                            IICA0 = g_ram_area[(gp_trx_data & RAM_MASK)];
                            gp_trx_data++;      /* move pointer to next address */
                            gp_trx_data |= RAM_BASE;
                        }

                    }
                }
            }
            else
            {

/*--------------------------------------
    data transfer timing
--------------------------------------*/

                if( RECEIVE_MODE == F_DIR ) /* check direction              */
                {

/*--------------------------------------
    receive timing
--------------------------------------*/

                    if( TRUTH == F_REGADR ) /* check reg addr timing        */
                    {

/*--------------------------------------
    register address receive timing
--------------------------------------*/

                        gp_trx_data = IICA0; /* set register address        */
                        F_REGADR = FALSE;   /* clear reg addr flag          */

                    }
                    else
                    {

/*--------------------------------------
    data receive timing
--------------------------------------*/

                        if ( RAM_BASE > gp_trx_data )
                        {
                            
                            g_rx_data[gp_trx_data] = IICA0; /* store received data */
                            gp_trx_data++;      /* move pointer to next addr    */
                            gp_trx_data &= RX_LIMIT;
                        }
                        else
                        {
/*--------------------------------------
    data receive and store it to RAM
--------------------------------------*/
                            g_ram_area[gp_trx_data & RAM_MASK] = IICA0;
                            gp_trx_data++;      /* move pointer to next addr    */
                            gp_trx_data |= RAM_BASE;
                        }

                    }
                    
                    WREL0 = TRUTH;          /* Cancel wait & start receive  */
                }
                else
                {

/*--------------------------------------
    data transmit timing
--------------------------------------*/

                    if( NACK == ACKD_f )    /* check communication end      */
                    {

/*--------------------------------------
    communication end(NACK from master)
--------------------------------------*/

                        g_status &= COM_FLAG_MASK; /* clear communication flag */
                        LREL0 = TRUTH;      /* exit from communication      */
                    }
                    else
                    {

/*--------------------------------------
    transmit next data to master
--------------------------------------*/
                        if ( RAM_BASE > gp_trx_data )
                        {


/*--------------------------------------
    transmit next A/D data to master
--------------------------------------*/
                            if ( TRUTH == g_low_data_index )
                            {               /* low byte transmit timing     */

                                IICA0 = g_low_data_temp; /* transmit low byte   */
                                g_low_data_index = FALSE;

                            }
                            else
                            {               /* high byte transmit timing    */

                                g_low_data_index = TRUTH; /* set low byet ready */
                                datawork = g_tx_data[gp_trx_data];
                                IICA0 = (datawork >> 8);  /* transmit high byte */
                                g_low_data_temp = datawork & 0xFF;
                                gp_trx_data++;
                                gp_trx_data &= TX_LIMIT;

                            }
                        }
                        else
                        {
/*--------------------------------------
    transmit next RAM data to master
--------------------------------------*/
                            IICA0 = g_ram_area[(gp_trx_data & RAM_MASK)];
                            gp_trx_data++;
                            gp_trx_data |= RAM_BASE;
                        }
                    }
                }
            }
        }
    }
    else
    {
        /* do noting */
    }

}/* End of interrupt function r_iica0_interrupt() */

